@@ -14,16 +14,36 @@ must be loaded.
 from shapely.geometry.base import BaseGeometry
 from shapely.geometry import box, Point, Polygon, MultiPolygon
 import shapely.wkt
-from shapely.ops import transform
+import shapely.wkb
 import geopandas as gpd
-import pandas as pd
-from sqlalchemy import create_engine
+from shapely.ops import transform
 from functools import lru_cache, partial
 from cached_property import cached_property_with_ttl
+import psycopg
+from psycopg_pool import ConnectionPool
 
 from mundipy.cache import (spatial_cache_footprint, pyproj_transform,
 	union_spatial_cache)
-from mundipy.geometry import from_dataframe, from_row_series
+from mundipy.geometry import from_dataframe, from_row_series, enrich_geom
+
+def elements_from_cursor(cur):
+	# get column names
+	colnames = [desc[0] for desc in cur.description]
+
+	return [ element_from_tuple(tup, colnames) for tup in cur.fetchall() ]
+
+def element_from_tuple(tup, colnames):
+	features = dict()
+	geom = None
+
+	for i, val in enumerate(tup):
+		# comes as WKB encoded
+		if colnames[i] == 'geometry':
+			geom = shapely.wkb.loads(bytes.fromhex(val))
+		else:
+			features[colnames[i]] = val
+
+	return enrich_geom(geom, features)
 
 class Dataset:
 	"""
@@ -32,7 +52,7 @@ class Dataset:
 	from mundipy.layer import Dataset
 
 	src = Dataset({
- 		'url': 'postgresql://postgres@localhost:5432/postgres',
+		'url': 'postgresql://postgres@localhost:5432/postgres',
 		'table': 'table_name'
 	})
 
@@ -48,24 +68,17 @@ class Dataset:
 		if isinstance(data, dict):
 			self._db_url = data['url']
 			self._db_table = data['table']
+
+			self._pool = ConnectionPool(self._db_url)
 		elif isinstance(data, str):
 			self.filename = data
 		else:
 			raise TypeError('data for Dataset() is neither filename nor dict with PostgreSQL details')
 
-	@cached_property_with_ttl(ttl=10)
-	def _conn(self):
-		"""SQLAlchemy connection to the database, if _db_url is set"""
-
-		if self._db_url is None:
-			raise TypeError('Dataset._conn called when no _db_url was set')
-
-		return create_engine(self._db_url)
-
 	@union_spatial_cache
 	def _load(self, geom, pcs='EPSG:4326'):
 		"""
-		Load part or the entire Dataset as a dataframe.
+		Load part or the entire Dataset as a list of mundipy geometries.
 
 		Takes geom as a shapely.geometry, or None to load the
 		entire dataset.
@@ -73,46 +86,31 @@ class Dataset:
 		Returns the dataset in PCS coordinates.
 		"""
 
-		# convert from WGS84 to local PCS
-		transformer = partial(transform, pyproj_transform('EPSG:4326', pcs))
-
 		if self._db_url is not None:
-			# no geom
-			if geom is None:
-				# build the query
-				query = "SELECT * FROM %s" % self._db_table
+			with self._pool.connection() as conn:
+				cur = conn.execute("SELECT * FROM %s" % self._db_table)
 
-				gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
-				gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
-				return gdf
+				# no geom
+				if geom is None:
+					# build the query
+					query = "SELECT * FROM %s" % self._db_table
+				else:
+					# load entire geometry
+					query = "SELECT * FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (self._db_table, geom.wkt)
 
-			# load entire geometry
-			query = "SELECT * FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (self._db_table, geom.wkt)
-
-			gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
-			gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
-			return gdf
+				elements = elements_from_cursor(conn.execute(query))
+				return [geo.transform('EPSG:4326', pcs) for geo in elements]
 
 		if geom is None:
 			gdf = gpd.read_file(self.filename)
 		else:
 			gdf = gpd.read_file(self.filename, bbox=geom)
 
-		gdf.set_geometry(gdf.geometry.apply(transformer), inplace=True, crs=pcs)
-		return gdf
-
-	@property
-	def dataframe(self):
-		"""Load an entire Dataset as a dataframe."""
-		return self._load(None)
-
-	@lru_cache(maxsize=8)
-	def local_dataframe(self, pcs):
-		return self._load(None, pcs=pcs)
+		return [geo.transform('EPSG:4326', pcs) for geo in from_dataframe(gdf)]
 
 	@lru_cache(maxsize=8)
 	def geometry_collection(self, pcs):
-		return from_dataframe(self.local_dataframe(pcs))
+		return self._load(None, pcs=pcs)
 
 	"""Read into a Dataset at a specific geometry (WGS84)."""
 	def inside_bbox(self, bbox, pcs='EPSG:4326'):
@@ -147,10 +145,10 @@ class LayerView:
 
 		`geom` - inherits from `shapely.geometry`
 
-        from mundipy.utils import plot
+		from mundipy.utils import plot
 
-        for feat in layer.intersects(Point(-37.0, 42.1)):
-            plot(feat)
+		for feat in layer.intersects(Point(-37.0, 42.1)):
+			plot(feat)
 		"""
 		if not isinstance(self.layer, Dataset):
 			raise TypeError('intersects() on not Dataset undefined')
@@ -162,8 +160,8 @@ class LayerView:
 		# buffer a little bit to prevent point
 		bbox = transform(to_wgs, geom.buffer(1e-8)).bounds
 
-		potentially_intersecting_gdf = self.layer.inside_bbox(bbox, self.pcs)
-		return from_dataframe(potentially_intersecting_gdf[potentially_intersecting_gdf.intersects(geom)])
+		potentially_intersecting = self.layer.inside_bbox(bbox, self.pcs)
+		return list(filter(lambda g: g.intersects(geom), potentially_intersecting))
 
 	def nearest(self, geom):
 		"""
@@ -187,17 +185,13 @@ class LayerView:
 			# buffer geom.bbox
 			bbox = transform(to_wgs, geom.buffer(buffer_size)).bounds
 
-			gdf = self.layer.inside_bbox(bbox, self.pcs)
-			if len(gdf) > 0:
-				res = min(gdf.iterrows(), key=lambda row: geom.distance(row[1].geometry))
-
-				return from_row_series(res[1])
+			items = self.layer.inside_bbox(bbox, self.pcs)
+			if len(items) > 0:
+				return min(items, key=lambda geo: geom.distance(geo))
 
 		# fuck it, check the whole dataframe
-		gdf = self.layer.local_dataframe(self.pcs)
-		if len(gdf) > 0:
-			res = min(gdf.iterrows(), key=lambda row: geom.distance(row[1].geometry))
-
-			return from_row_series(res[1])
+		items = self.layer.geometry_collection(self.pcs)
+		if len(items) > 0:
+			return min(items, key=lambda geo: geom.distance(geo))
 
 		return None
